;| TextIntegerIncrement.lsp [command name: TII]
To change the text content of existing successively User-selected Text/Mtext/MultiLeader
  objects or non-constant Block Attributes to upward-incrementing integer values.  Replaces
  entire textual content with text representing next integer, regardless of type or length of
  original content.  Layer, Style, Height, Rotation, Justification and other properties remain.
User specifies starting value [initial default "1"], which can include any number of leading
  0's, and routine counts upward from there by 1 with each successful selection, when
  appropriate "padding" with leading 0(s) for same number of digits.
MultiLeader can be selected on either Mtext part or Leader part; Attribute in Block must be
  selected on Attribute itself [not any other part of Block], and must not be constant-value.
Undo option within command steps back last change and increments content value back
  down, repeatedly if desired as long as there are changes to undo. Undo after end of
  command undoes all changes made within it, collectively.
Remembers number reached [and if using leading 0(s) how many digits to pad out to], and
  offers as defaults on subsequent use, to continue from there.
Kent Cooper, 22 January 2020
|;

(defun C:TII (/ *error* doc TIIstr n tsel tent tobj tnam done); = Text Integer Increment

  (defun *error* (errmsg)
    (if (not (wcmatch errmsg "Function cancelled,quit / exit abort,console break"))
      (princ (strcat "\nError: " errmsg))
    ); if
    (vla-endundomark doc)
    (princ)
  ); defun - *error*
  (vla-startundomark (setq doc (vla-get-activedocument (vlax-get-acad-object))))

  (defun 0pad (/ out)
    (setq out (itoa *TIIint))
    (while (< (strlen out) *TIIdig)
      (setq out (strcat "0" out))
    ); while
    out
  ); defun

  (while
    (not
      (and
        (not (initget 4)); no negative
        (setq
          TIIstr
          (getstring ; to retain leading zero(s) entered, if any [lost with (getint)]
            (strcat
              "\nStarting integer value {can include leading 0(s)} <"
              (cond (*TIIdig (0pad)) (*TIIint (itoa *TIIint)) ("1"))
                ;; [*TIIdig will not exist without *TIIint]
              "> :"
            ); strcat
          ); getstring
          n 0 ; number of selections/content changes
        ); setq
        (= (vl-string-trim "0123456789" TIIstr) "") ;;;;; allow negative?
          ; represents integer [with or without leading zero(s)], or Enter/default
      ); and
    ); not
    (prompt "\nRequires integer value.")
  ); while

  (cond
    ((wcmatch TIIstr "0*"); User input w/ leading 0(s)
      (setq ; then
        *TIIdig (strlen TIIstr); number of digits
        *TIIint (atoi TIIstr); as integer [drops leading zero(s)]
      ); setq
    ); input-with-leading-zero(s) condition
    ((/= TIIstr ""); typed input without leading zero(s)
      (setq
        *TIIdig nil ; cancel 0-padding if previously used
        *TIIint (atoi TIIstr); as integer
      ); setq
    ); input-without-leading-zero(s) condition
    ((not *TIIint) (setq *TIIint 1)); first-use default
    ;; [on further Enter/default, keep integer & 0-padding if previously used]
  ); cond

  (while (not done)
    (setvar 'errno 0)
    (if (> n 0) (initget "Undo")); Undo option only if things to undo
    (setq tsel
      (entsel
        (strcat
          "\nText/Mtext/MLeader/Attribute to change content to "
          (if *TIIdig (0pad) (itoa *TIIint))
          " or "
          (if (> n 0) "[Undo] " ""); Undo option only if things to undo
          "<exit>: "
        ); strcat
      ); entsel
    ); setq
    (cond ; object-selection/Undo/missed-pick/exit
      ((= tsel "Undo")
        (setq n (1- n)); for quantity changed / Undo option
        (setq *TIIint (1- *TIIint)); for content
        (command "_undo" "_back"); revert last selected object to original content
      ); Undo condition
      ( ; test for selection, valid entity type, unlocked Layer
        (and
          (= (getvar 'errno) 0); picked something
          (setq
            tent (car tsel)
            tobj (vlax-ename->vla-object tent)
            tnam (substr (vla-get-ObjectName tobj) 5); without AcDb prefix
          ); setq
          (= (logand 4 (cdr (assoc 70 (tblsearch "layer" (vla-get-Layer tobj))))) 0)
            ; 0 = unlocked Layer, 4 = locked
          (wcmatch tnam "Text,MText,MLeader,BlockReference")
            ; [not *Text because of RText possibility]
          (cond
            ((wcmatch tnam "Text,MText,MLeader")); any OK
            ( ; a non-constant Attribute in a Block? make that tobj
              (and
                (= tnam "BlockReference")
                (not (assoc 1 (tblsearch "block" (vla-get-effectivename tobj))))
                  ; = not an Xref
                (= (cdr (assoc 0 (entget (setq tent (car (nentselp (cadr tsel))))))) "ATTRIB")
                  ; tent so tobj is left alone if not Attribute
                  ; [constant Attribute's entity type is "ATTDEF"]
              ); and
              (setq tobj (vlax-ename->vla-object tent)); replace parent Block with Attribute
            ); non-constant Attribute condition
          ); cond [returns nil for non-Attribute or constant-Attribute part of Block]
        ); and
        (command "_.undo" "_mark")
        (vla-put-TextString tobj (if *TIIdig (0pad) (itoa *TIIint))); impose content
        (setq n (1+ n)); for Undo option
        (setq *TIIint (1+ *TIIint)); for content
      ); picked-something-valid condition
      ((= (getvar 'errno) 0); picked something, but invalid under (and) tests above
        (prompt "\nInapplicable object, constant-value Attribute, or on a locked Layer --")
      ); non-qualifying condition
      ((= (getvar 'errno) 7) (prompt "\nNothing selected --")); missed
      ((setq done T)); Enter/space at Select-object prompt [errno = 52]; stop (while) loop
    ); cond -- selection or U
  ); while

  (vla-endundomark doc)
  (princ)
); defun

(vl-load-com)
(prompt "\nType TII for Text Integer Increment routine.")
(princ)